#!/usr/bin/perl
package HelpFuncs;
# Port to Unix

use  Logging;


my $logFileName = undef;
my $dumpFileName = undef;
my $statusFileName = undef;

sub set_log_file{
  $logFileName = $_[0];
  unlink $logFileName if $logFileName and -e $logFileName;
}




sub log_debug{
  Logging::debug( "$_[0] ( $_[1] )" );
}

sub log_error{
  Logging::error( "$_[0] ( $_[1] )" );
}

sub log_admin{
  Logging::info( "$_[0] ( $_[1] )" );
}

sub log_info{
  Logging::info( "$_[0] ( $_[1] )" );
}



sub set_dump_file{
 #nothing to do linux
}

sub set_status_file{
 #nothing to do linux
}

sub write_status
{
 #nothing to do linux
}

sub negation{
	my ($val) = @_;
	if ($val eq 'true') {
		return 'false';
	}
	if ($val eq 'false') {
		return 'true'; 
	}
}

sub checkValue{
	my ($val, $defaultVal, $checkVal) = @_;
	if( defined $val ) {
		if ( defined $checkVal ) {
			return $checkVal if $val eq $checkVal;
		}
		else {
			return $val;
		}
	} 
	return $defaultVal;
}

sub dieWithError{
  my( $msg, $errCode ) = @_;
  $errCode = -1 if not $errCode;
  log_error( "The program faield with error $errCode", $msg );
  my $idx = 1;
  my $package = 1;
  my $tab = ' ';
  my $callstack = '';
  while( $package and $idx<15 ) {
      my  ($package, $filename, $line, $subroutine, $hasargs, $wantarray, $evaltext, $is_require, $hints, $bitmask) = caller($idx);
      if( $package ) {
          $tab = $tab . ' ';
          $callstack .= "\n$tab Stack[$idx]. Sub:$subroutine [Package:$package][Filename:$filename]Line:$line";
      }
     $idx = $idx + 1;
    }
  die "\nDie with error:$msg\nCall stack:\n$callstack";
}

sub randomPasswd {
  my $len = shift;
  if ( $len <= 0 ) {
    $len = 8;
  }
  my $symbols = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
  my $seq_length = length($symbols);
  my $passwd = '';

  for (my $i = 0; $i <= $len; ++$i) {
    $passwd .= substr($symbols, rand($seq_length), 1);
  }

  return $passwd;
}

sub getUniqueDir{
  my( $path, $prefix ) = @_;
  my $cnt = 1;
  while( -e "$path/$prefix\[$cnt\]" ){ $cnt += 1; }
  return "$path/$prefix\[$cnt\]";
}

sub deleteFolder{
  my( $path ) = @_;
  opendir( DH, $path );
  my $dir;
  while( ( $dir = readdir( DH ) ) ){
    if( not $dir eq '.' and not $dir eq '..' ){
      if( -d "$path/$dir" ){
        deleteFolder( "$path/$dir" );
        if( not rmdir "$path/$dir" ) { 
          log_error "Cannot delete directory $path/$dir"; }
        }
      else{ unlink "$path/$dir" or log_error "Cannot delete file '$path/$dir'"; }
    }
  }
  closedir( DH );
  if( not rmdir $path ) {
    log_error "Cannot delete directory $path"; 
  }
}

sub make_path {
  my $arg = {};
  if (@_ and (ref($_[-1]) =~ /HASH/) ) {
    $arg = pop @_;
  }
  if (exists $arg->{owner} and $arg->{owner} =~ /\D/) {
    my $uid = (getpwnam $arg->{owner})[2];
    if (defined $uid) {
      $arg->{owner} = $uid;
    }
    else {
      log_error "Unable to map $arg->{owner} to a uid, ownership not changed";
      delete $arg->{owner};
    }
  }
  if (exists $arg->{group} and $arg->{group} =~ /\D/) {
    my $gid = (getgrnam $arg->{group})[2];
    if (defined $gid) {
      $arg->{group} = $gid;
    }
    else {
      log_error "Unable to map $arg->{group} to a gid, group ownership not changed";
      delete $arg->{group};
    }
  }
  if (exists $arg->{owner} and not exists $arg->{group}) {
    $arg->{group} = -1; # chown will leave group unchanged
  }
  if (exists $arg->{group} and not exists $arg->{owner}) {
    $arg->{owner} = -1; # chown will leave owner unchanged
  }
  push @_, $arg;
  _make_path(@_);
}

sub _make_path {
  my $arg = pop @_;
  my @paths = @_;
  foreach $path (@paths) {
    next unless defined($path) and length($path);
    next if -d $path;
    my @dirs = split ('/', $path);
    pop @dirs;
    my $parent = join('/',@dirs);
    unless (-d $parent or $path eq $parent) {
      _make_path( $parent, $arg);
    }
    if (mkdir $path) {
      if( exists $arg->{owner} and exists $arg->{group}) {
        if (!chown 0+$arg->{owner}, 0+$arg->{group}, $path) {
          log_error "Cannot chown on '$path' to '$arg->{owner}:$arg->{group}'";
        }
      }
      if (exists $arg->{mode}) {
        if (!chmod $arg->{mode}, $path) {
          log_error "Cannot chmod on '$path' to $arg->{mode}";
        }
      }
    }
    else {
      log_error "Cannot mkdir '$path'";
    }
  }
}

sub epoch2CrDate {
  my $epoch = shift;
  unless ( $epoch ) {
    return '1970-01-01';
  }
  my ($mday, $mon, $year) = (localtime($epoch)) [3 .. 5];
  return sprintf("%04d-%02d-%02d", $year+1900, $mon+1, $mday);
}

sub getRelativePath {
  my ($subpath, $path) = @_;
  if ( !$subpath || !$path) {
    return;
  }
  $subpath =~ /^($path)\/*(.*)/;
  if ( defined $2 ) {
    return $2;
  }
  return;
}


1;